@@ -5,6 +5,7 @@ use SG\SgEstateCore\Domain\Model\Badausstattung;
 use SG\SgEstateCore\Domain\Model\Filter;
 use SG\SgEstateCore\Domain\Model\Heizungsart;
 use SG\SgEstateCore\Domain\Model\Immobilie;
+use SG\SgEstateCore\Domain\Model\Kontaktperson;
 use SG\SgEstateCore\Domain\Model\Objektart;
 use SG\SgEstateCore\Domain\Model\Objektarttyp;
 use SG\SgEstateCore\Domain\Model\Ort;
@@ -21,6 +22,7 @@ use SG\SgEstateCore\Util\Services;
 use SG\SgEstateCore\Util\SessionHandler;
 use TYPO3\CMS\Core\Cache\CacheManager;
 use TYPO3\CMS\Core\Cache\Frontend\FrontendInterface;
+use TYPO3\CMS\Core\Core\Environment;
 use TYPO3\CMS\Core\Mail\MailMessage;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
@@ -28,11 +30,14 @@ use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
 use TYPO3\CMS\Extbase\Mvc\Exception\NoSuchArgumentException;
 use TYPO3\CMS\Extbase\Mvc\Exception\StopActionException;
 use TYPO3\CMS\Extbase\Mvc\Exception\UnsupportedRequestTypeException;
+use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
 use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
+use TYPO3\CMS\Extbase\Service\ImageService;
 use TYPO3\CMS\Extbase\Utility\DebuggerUtility;
 use TYPO3\CMS\Extbase\Utility\LocalizationUtility;
 use TYPO3\CMS\Extbase\Validation\Validator\NotEmptyValidator;
 use TYPO3\CMS\Fluid\View\StandaloneView;
+use TYPO3\CMS\Core\Context\Context;
 
 class BaseController extends ActionController {
 
@@ -144,6 +149,30 @@ class BaseController extends ActionController {
         $this->sgEstateCoreServices = $services;
     }
 
+    /**
+     * @var ImageService
+     */
+    protected $imageService;
+
+    /**
+     * @param ImageService $imageService
+     */
+    public function injectImageService(ImageService $imageService){
+        $this->imageService = $imageService;
+    }
+
+    /**
+     * @var UriBuilder
+     */
+    protected $uriBuilder;
+
+    /**
+     * @param UriBuilder $uriBuilder
+     */
+    public function injectUriBuilder(UriBuilder $uriBuilder){
+        $this->uriBuilder = $uriBuilder;
+    }
+
     /**
      * @var Filter
      */
@@ -176,11 +205,9 @@ class BaseController extends ActionController {
     public function searchAction(){
 		
 		// this fix multilanguge sites 
-		if(isset($this->getLanguageUid()) && null !== $this->getLanguageUid()) {
-			$cacheIdentifier = 'search_'.$this->settings['search']['layout'].'_'.implode('', $this->settings['show']).'_'.$this->settings['search']['layout'].'_L'.$this->getLanguageUid();
-		} else {
-			$cacheIdentifier = strtolower($this->settings['search']['layout']).'_search_' . $this->filter->getCacheIdentifier();
-		}
+		$cacheIdentifier = 'search_'.$this->settings['search']['layout'].'_'.implode('', $this->settings['show']).'_'.$this->settings['search']['layout'].'_L'.$this->getLanguageUid();
+		//$cacheIdentifier = strtolower($this->settings['search']['layout']).'_search_' . $this->filter->getCacheIdentifier();
+	
         
         if (($entry = $this->cache->get($cacheIdentifier)) === FALSE) {
             $kindOfObjectUids = explode(',', $this->settings['search']['kindOfObjects']);
@@ -290,6 +317,8 @@ class BaseController extends ActionController {
             $this->view->assign('cityOptions', $this->ortRepository->getAllForFilter($arrayOfImportIds));
             $this->view->assign('groupedCitiesAndDistricts', $this->ortRepository->getGroupedCitiesAndDistrictsForFilter($arrayOfImportIds));
             $this->view->assign('filter', $this->filter);
+            $this->view->assign('aggregatedFilterData', json_encode($this->getAggregatedFilterData()));
+            $this->view->assign('citiesAndDistrictsForFilter', json_encode($this->generateCityAndDistrictStructureForFilter()));
 
             $entry = $this->view->render();
 
@@ -382,6 +411,36 @@ class BaseController extends ActionController {
         }
     }
 
+    private function generateCityAndDistrictStructureForFilter(){
+        $cityAndDistrictStructure = [];
+
+        foreach ($this->ortRepository->findAll() as $city){
+            /**
+             * @var $city Ort
+             */
+            $newCity = [
+                'caption' => $city->getBezeichner(),
+                'uid' => $city->getUid()
+            ];
+            if (count($city->getStadtteile()) > 0){
+                $newDistricts = [];
+                foreach ($city->getStadtteile() as $district){
+                    /**
+                     * @var $district Stadtteil
+                     */
+                    $newDistrict = [
+                        'caption' => $district->getBezeichner(),
+                        'uid' => $district->getUid()
+                    ];
+                    $newDistricts[] = $newDistrict;
+                }
+                $newCity['districts'] = $newDistricts;
+            }
+            $cityAndDistrictStructure[] = $newCity;
+        }
+        return $cityAndDistrictStructure;
+    }
+
     /**
      * Action used for several AJAX Calls
      *
@@ -397,6 +456,75 @@ class BaseController extends ActionController {
             $function = $this->request->getArgument('function');
             $this->view->assign('function', ucfirst($function));
             switch ($function){
+                case 'realtiesForReactApplication':
+                    $cacheIdentifier = 'ajax_realtiesForReactApplication';
+                    if (($entry = $this->cache->get($cacheIdentifier)) === FALSE) {
+                        $allRealties = $this->immobilieRepository->findAll();
+                        $arrayReturn = array(
+                            'realties' => array()
+                        );
+                        $processingInstructions = array(
+                            'width' => '150c',
+                            'height' => '112c',
+                        );
+                        foreach ($allRealties as $realty){
+                            /**
+                             * @var $realty Immobilie
+                             */
+                            $district = array();
+                            if( $realty->getObjektStadtteil() instanceof Stadtteil )
+                            {
+                                $district = array(
+                                    'id' => $realty->getObjektStadtteil()->getUid(),
+                                    'name' => $realty->getObjektStadtteil()->getBezeichner()
+                                );
+                            }
+                            $image = $this->imageService->getImage(Environment::getPublicPath().'/uploads/tx_sgestatecore/media/'.$realty->getTitelbild()->getDatei(),null,false);
+                            $processedImage = $this->imageService->applyProcessingInstructions($image, $processingInstructions);
+
+                            array_push($arrayReturn['realties'], array(
+                                'show' => false,
+                                'highlighted' => false,
+                                'id' => $realty->getUid(),
+                                'data' => [
+                                    'title' => $realty->getObjekttitel(),
+                                    'street' => $realty->getObjektStrasse(),
+                                    'houseNumber' => $realty->getObjektHausnummer(),
+                                    'zip' => $realty->getObjektPlz(),
+                                    'size' => $realty->getWohnflaeche(),
+                                    'rooms' => $realty->getAnzahlZimmer(),
+                                    'rent' => $realty->getNettokaltmiete(),
+                                    'balcony' => $realty->getAnzahlBalkone() > 0 ? true : false,
+                                    'city' => array(
+                                        'id' => $realty->getObjektOrt()->getUid(),
+                                        'name' => $realty->getObjektOrt()->getBezeichner()
+                                    ),
+                                    'district' => $district,
+                                    'url' => $this->uriBuilder->reset()->setTargetPageUid($this->settings['pid']['detail'])->uriFor('detail', ['realty' => $realty->getUrlIdentifier()], 'Base', 'sgestatebase', 'pi1')
+                                ],
+                                'media' => [
+                                    'preview' => $this->imageService->getImageUri($processedImage)
+                                ],
+                                'geometry' => [
+                                    'location' => [
+                                        'lat' => $realty->getObjektBreitengrad(),
+                                        'lng' => $realty->getObjektLaengengrad()
+                                    ]
+                                ],
+                            ));
+                        }
+                        $this->view->assign('jsonReturn', json_encode($arrayReturn));
+
+                        $entry = $this->view->render();
+
+                        $tags = array('ajax_realtiesForReactApplication');
+                        $lifetime = $this->settings['defaultCacheLifetime'];
+
+                        // Save value in cache
+                        $this->cache->set($cacheIdentifier, $entry, $tags, $lifetime);
+                    }
+                    return $entry;
+                    break;
                 case 'allRealties':
                     $cacheIdentifier = 'ajax_allRealties';
 
@@ -577,6 +705,24 @@ class BaseController extends ActionController {
                         }
                     }
                     break;
+                case 'getCityAndDistricts':
+                    $cacheIdentifier = 'ajax_getCityAndDistricts';
+                    if (($entry = $this->cache->get($cacheIdentifier)) === FALSE) {
+                        $cityAndDistrictStructure = $this->generateCityAndDistrictStructureForFilter();
+
+                        $this->view->assign('jsonReturn', $cityAndDistrictStructure);
+
+                        $entry = $this->view->render();
+
+
+                        $tags = array('ajax_getCityAndDistricts');
+                        $lifetime = $this->settings['defaultCacheLifetime'];
+
+                        // Save value in cache
+                        $this->cache->set($cacheIdentifier, $entry, $tags, $lifetime);
+                    }
+                    return $entry;
+                    break;
                 default:
                     break;
             }
@@ -650,16 +796,21 @@ class BaseController extends ActionController {
         if ($valid){
             // Generate Receivers
             if ($this->settings['contact']['useRealtyContactAsReceiver'] == 1){
-                $emailDirekt = $realty->getKontaktperson()->getEmailDirekt();
-                $emailZentrale = $realty->getKontaktperson()->getEmailZentrale();
-                $emailSonstige = $realty->getKontaktperson()->getEmailSonstige();
-                if (GeneralUtility::validEmail($emailDirekt)){
-                    $receivers = array($emailDirekt);
-                }elseif (GeneralUtility::validEmail($emailZentrale)){
-                    $receivers = array($emailZentrale);
-                }elseif (GeneralUtility::validEmail($emailSonstige)){
-                    $receivers = array($emailSonstige);
-                }else{
+                if( $realty->getKontaktperson() instanceof Kontaktperson ) {
+                    $emailDirekt = $realty->getKontaktperson()->getEmailDirekt();
+                    $emailZentrale = $realty->getKontaktperson()->getEmailZentrale();
+                    $emailSonstige = $realty->getKontaktperson()->getEmailSonstige();
+                    if (GeneralUtility::validEmail($emailDirekt)){
+                        $receivers = array($emailDirekt);
+                    }elseif (GeneralUtility::validEmail($emailZentrale)){
+                        $receivers = array($emailZentrale);
+                    }elseif (GeneralUtility::validEmail($emailSonstige)){
+                        $receivers = array($emailSonstige);
+                    }else{
+                        $receivers = explode(',',$this->settings['contact']['receivers']);
+                    }
+                }
+                else {
                     $receivers = explode(',',$this->settings['contact']['receivers']);
                 }
             }else{
@@ -1129,4 +1280,27 @@ class BaseController extends ActionController {
         $message->send();
         return $message->isSent();
     }
+    
+    
+    /**
+	 * Get the current language
+	 */
+	private function getLanguageUid() {
+		$context = GeneralUtility::makeInstance(Context::class);
+		$sysLanguageUid = $context->getPropertyFromAspect('language', 'id');
+		if (is_numeric($sysLanguageUid)) {
+			return $sysLanguageUid;
+		} else {
+			return 0;
+		}
+	}
+
+	private function getAggregatedFilterData(){
+        return [
+            'size' => !$this->filter->getFlaecheAb() ? $this->getSizeSliderData()['value'] : $this->filter->getFlaecheAb(),
+            'rooms' => !$this->filter->getRaeumeAb() ? $this->getRoomSliderData()['value'] : $this->filter->getRaeumeAb(),
+            'rent' => !$this->filter->getKaltmieteBis() ? $this->getAmountSliderData()['value'] : $this->filter->getKaltmieteBis()
+        ];
+    }
+
 }
